@@ -5,7 +5,7 @@ import argparse
 import cli_ui
 import logging
 import luddite
-import pkg_resources
+from importlib.metadata import version as package_version
 import textwrap
 import traceback
 from cli_ui import (
@@ -25,7 +25,7 @@ from cli_ui import (
     purple,
     warning,
 )
-from packaging import version as packaging_version
+from packaging import version
 from typing import Any, Tuple
 
 from gitlabform.configuration import Configuration
@@ -42,12 +42,20 @@ from gitlabform.lists.filter import GroupsAndProjectsFilters
 from gitlabform.lists.groups import GroupsProvider
 from gitlabform.lists.projects import ProjectsProvider
 from gitlabform.output import EffectiveConfigurationFile
+from gitlabform.processors.application import ApplicationProcessors
 from gitlabform.processors.group import GroupProcessors
 from gitlabform.processors.project import ProjectProcessors
 
 
 class GitLabForm:
-    def __init__(self, include_archived_projects=True, target=None, config_string=None):
+    def __init__(
+        self,
+        include_archived_projects=True,
+        target=None,
+        config_string=None,
+        noop=False,
+        output_file=None,
+    ):
         if target and config_string:
             # this mode is basically only for testing
 
@@ -58,8 +66,8 @@ class GitLabForm:
             self.strict = True
             self.start_from = 1
             self.start_from_group = 1
-            self.noop = False
-            self.output_file = None
+            self.noop = noop
+            self.output_file = output_file
             self.skip_version_check = True
             self.include_archived_projects = include_archived_projects
             self.just_show_version = False
@@ -101,6 +109,9 @@ class GitLabForm:
 
         self.gitlab, self.configuration = self._initialize_configuration_and_gitlab()
 
+        self.application_processors = ApplicationProcessors(
+            self.gitlab, self.configuration, self.strict
+        )
         self.group_processors = GroupProcessors(
             self.gitlab, self.configuration, self.strict
         )
@@ -367,6 +378,16 @@ class GitLabForm:
 
         effective_configuration = EffectiveConfigurationFile(self.output_file)
 
+        application_configuration = self.configuration.get("application", {})
+        if application_configuration:
+            self.application_processors.process_entity(
+                "",
+                application_configuration,
+                dry_run=self.noop,
+                effective_configuration=effective_configuration,
+                only_sections=self.only_sections,
+            )
+
         for group in groups:
             group_number += 1
 
@@ -469,6 +490,12 @@ class GitLabForm:
                 successful_projects += 1
 
             except Exception as e:
+                if "Non GET methods are not allowed for moved projects" in str(e):
+                    info(
+                        "Project has been transferred, no need to process original location"
+                    )
+                    continue
+
                 failed_projects[project_number] = project_and_group
 
                 trace = traceback.format_exc()
@@ -508,7 +535,7 @@ class GitLabForm:
         :param skip_version_check: if True then the comparison to the latest versions is skipped
         """
 
-        local_version = pkg_resources.get_distribution("gitlabform").version
+        local_version = package_version("gitlabform")
 
         # fmt: off
         tower_crane = Symbol("🏗", "")
@@ -533,9 +560,7 @@ class GitLabForm:
                 happy = Symbol("😊", ":)")
                 to_show = ["= the latest stable ", happy]
                 # fmt: on
-            elif packaging_version.parse(local_version) < packaging_version.parse(
-                latest_version
-            ):
+            elif version.parse(local_version) < version.parse(latest_version):
                 # fmt: off
                 sad = Symbol("😔", ":(")
                 to_show = ["= outdated ", sad, " , please update! (the latest stable is ", latest_version, ")"]
@@ -562,9 +587,11 @@ class GitLabForm:
         """
 
         if target == "ALL":
-            info(">>> Getting ALL groups and projects...")
+            info(
+                ">>> Getting ALL groups and projects that I have permission to modify..."
+            )
         elif target == "ALL_DEFINED":
-            info(">>> Getting all groups and projects defined in the configuration...")
+            info(">>> Getting ALL groups and projects DEFINED in the configuration...")
         else:
             info(">>> Getting requested groups or projects...")
 
