@@ -15,12 +15,13 @@ from gitlabform.gitlab.project_deploy_keys import GitLabProjectDeployKeys
 from gitlabform.gitlab.project_protected_environments import (
     GitLabProjectProtectedEnvironments,
 )
+from gitlabform.gitlab.project_merge_requests_approvals import (
+    GitLabProjectMergeRequestsApprovals,
+)
+from gitlabform.gitlab.python_gitlab import PythonGitlab
 from gitlabform.gitlab.variables import GitLabVariables
 from gitlabform.gitlab.repositories import GitLabRepositories
 from gitlabform.gitlab.resource_groups import GitLabResourceGroups
-from gitlabform.gitlab.schedules import GitLabPipelineSchedules
-from gitlabform.gitlab.integrations import GitLabIntegrations
-from gitlabform.gitlab.tags import GitLabTags
 from gitlabform.gitlab.users import GitLabUsers
 
 
@@ -55,18 +56,38 @@ class GitLab(
     GitLabMergeRequests,
     GitLabRepositories,
     GitLabResourceGroups,
-    GitLabIntegrations,
-    GitLabTags,
     GitLabGroupLDAPLinks,
     GitLabGroupBadges,
     GitLabGroupVariables,
     GitLabPipelines,
     GitLabMembers,
     GitLabUsers,
-    GitLabPipelineSchedules,
     GitLabProjectBadges,
     GitLabProjectDeployKeys,
-    GitLabVariables,
     GitLabProjectProtectedEnvironments,
+    GitLabProjectMergeRequestsApprovals,
+    GitLabVariables,
 ):
     pass
+
+
+class GitlabWrapper:
+    def __init__(self, gitlabform: GitLab):
+        url = gitlabform.url
+        token = gitlabform.token
+        ssl_verify = gitlabform.ssl_verify
+        timeout = gitlabform.timeout
+        session = gitlabform.session
+
+        self._gitlab: PythonGitlab = PythonGitlab(
+            url,
+            token,
+            ssl_verify=ssl_verify,
+            api_version="4",
+            session=session,
+            retry_transient_errors=True,
+            timeout=timeout,
+        )
+
+    def get_gitlab(self):
+        return self._gitlab
