@@ -3,12 +3,12 @@ import os
 from logging import debug
 from urllib import parse
 
-import pkg_resources
+from importlib.metadata import version as package_version
 import requests
 
 # noinspection PyPackageRequirements
 import urllib3
-from cli_ui import debug as verbose
+from cli_ui import debug as verbose, warning
 from requests.adapters import HTTPAdapter
 
 # noinspection PyPackageRequirements
@@ -42,8 +42,8 @@ class GitLabCore:
         if not self.ssl_verify:
             urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
-        self.gitlabform_version = pkg_resources.get_distribution("gitlabform").version
-        self.requests_version = pkg_resources.get_distribution("requests").version
+        self.gitlabform_version = package_version("gitlabform")
+        self.requests_version = package_version("requests")
         self.session.headers.update(
             {
                 "private-token": self.token,
@@ -58,6 +58,18 @@ class GitLabCore:
                 f"Connected to GitLab version: {version['version']} ({version['revision']})"
             )
             self.version = version["version"]
+
+            current_user = self._make_requests_to_api("user")
+            if current_user.get("is_admin", False):
+                self.admin = True
+            else:
+                self.admin = False
+            verbose(
+                f"Connected as: {current_user['username']}, admin: {'yes' if self.admin else 'no'}"
+            )
+            if not self.admin:
+                warning("Connected as non-admin. You may encounter permission issues.")
+
         except Exception as e:
             raise TestRequestFailedException(e)
 
@@ -99,10 +111,6 @@ class GitLabCore:
         result = self.get_project(project_and_group)
         return str(result["id"])
 
-    def has_no_license(self):
-        license = self._make_requests_to_api("license")
-        return not license or license["expired"]
-
     def _make_requests_to_api(
         self,
         path_as_format_string,
