@@ -1,4 +1,4 @@
-from logging import debug
+from logging import debug, warning
 from typing import Dict, Tuple
 
 from cli_ui import fatal
@@ -6,13 +6,17 @@ from cli_ui import fatal
 from gitlabform.constants import EXIT_INVALID_INPUT
 from gitlabform.gitlab import GitLab, AccessLevel
 from gitlabform.processors.abstract_processor import AbstractProcessor
+from gitlab.v4.objects import Group, GroupMember
+from gitlab import GitlabDeleteError
 
 
 class GroupMembersProcessor(AbstractProcessor):
     def __init__(self, gitlab: GitLab):
         super().__init__("group_members", gitlab)
 
-    def _process_configuration(self, group: str, configuration: dict):
+    def _process_configuration(self, group_name: str, configuration: dict):
+        keep_bots = configuration.get("group_members|keep_bots", False)
+
         enforce_group_members = configuration.get("group_members|enforce", False)
 
         (
@@ -32,9 +36,16 @@ class GroupMembersProcessor(AbstractProcessor):
                 exit_code=EXIT_INVALID_INPUT,
             )
 
+        group = self.gl.get_group_by_path_cached(group_name)
+
         self._process_groups(group, groups_to_set_by_group_path, enforce_group_members)
 
-        self._process_users(group, users_to_set_by_username, enforce_group_members)
+        self._process_users(
+            users_to_set_by_username,
+            enforce_group_members,
+            keep_bots,
+            group,
+        )
 
     @staticmethod
     def _get_groups_and_users_to_set(configuration: dict) -> Tuple[dict, dict]:
@@ -51,32 +62,33 @@ class GroupMembersProcessor(AbstractProcessor):
                 users_to_set_by_username.pop("enforce", None)
                 users_to_set_by_username.pop("users", None)
                 users_to_set_by_username.pop("groups", None)
+                users_to_set_by_username.pop("keep_bots", None)
 
         return groups_to_set_by_group_path, users_to_set_by_username
 
     def _process_groups(
-        self, group: str, groups_to_set_by_group_path: dict, enforce_group_members: bool
+        self,
+        group_being_processed: Group,
+        groups_to_share_with_by_path: dict,
+        enforce_group_members: bool,
     ):
-        # group users before by group name
-        groups_before = self.gitlab.get_group_case_insensitive(group)[
-            "shared_with_groups"
-        ]
-        debug("Group shared with BEFORE: %s", groups_before)
+        shared_with_groups_before = group_being_processed.shared_with_groups
+        debug("Group shared with BEFORE: %s", shared_with_groups_before)
 
         groups_before_by_group_path = dict()
-        for share_details in groups_before:
-            groups_before_by_group_path[
-                share_details["group_full_path"]
-            ] = share_details
+        for shared_with_group in shared_with_groups_before:
+            groups_before_by_group_path[shared_with_group["group_full_path"]] = (
+                shared_with_group
+            )
 
-        for share_with_group_path in groups_to_set_by_group_path:
-            group_access_to_set = groups_to_set_by_group_path[share_with_group_path][
+        for share_with_group_path in groups_to_share_with_by_path:
+            group_access_to_set = groups_to_share_with_by_path[share_with_group_path][
                 "group_access"
             ]
 
             expires_at_to_set = (
-                groups_to_set_by_group_path[share_with_group_path]["expires_at"]
-                if "expires_at" in groups_to_set_by_group_path[share_with_group_path]
+                groups_to_share_with_by_path[share_with_group_path]["expires_at"]
+                if "expires_at" in groups_to_share_with_by_path[share_with_group_path]
                 else None
             )
 
@@ -101,14 +113,15 @@ class GroupMembersProcessor(AbstractProcessor):
                         "Re-adding group '%s' to change their access level or expires at.",
                         share_with_group_path,
                     )
+                    share_with_group_id = groups_before_by_group_path[
+                        share_with_group_path
+                    ]["group_id"]
                     # we will remove the group first and then re-add them,
                     # to ensure that the group has the expected access level
-                    self.gitlab.remove_share_from_group(group, share_with_group_path)
-                    self.gitlab.add_share_to_group(
-                        group,
-                        share_with_group_path,
-                        group_access_to_set,
-                        expires_at_to_set,
+                    self._unshare(group_being_processed, share_with_group_id)
+
+                    group_being_processed.share(
+                        share_with_group_id, group_access_to_set, expires_at_to_set
                     )
 
             else:
@@ -116,36 +129,52 @@ class GroupMembersProcessor(AbstractProcessor):
                     "Adding group '%s' who previously was not a member.",
                     share_with_group_path,
                 )
-                self.gitlab.add_share_to_group(
-                    group, share_with_group_path, group_access_to_set, expires_at_to_set
+
+                share_with_group_id = self.gl.get_group_id(share_with_group_path)
+                group_being_processed.share(
+                    share_with_group_id, group_access_to_set, expires_at_to_set
                 )
 
         if enforce_group_members:
             # remove groups not configured explicitly
             groups_not_configured = set(groups_before_by_group_path) - set(
-                groups_to_set_by_group_path
+                groups_to_share_with_by_path
             )
             for group_path in groups_not_configured:
                 debug(
                     "Removing group '%s' who is not configured to be a member.",
                     group_path,
                 )
-                self.gitlab.remove_share_from_group(group, group_path)
+                share_with_group_id = self.gl.get_group_id(group_path)
+                self._unshare(group_being_processed, share_with_group_id)
         else:
             debug("Not enforcing group members.")
 
-        debug("Group shared with AFTER: %s", self.gitlab.get_group_members(group))
+        debug(
+            "Group shared with AFTER: %s",
+            group_being_processed.members.list(get_all=True),
+        )
+
+    @staticmethod
+    def _unshare(group_being_processed, share_with_group_id):
+        try:
+            group_being_processed.unshare(share_with_group_id)
+        except GitlabDeleteError:
+            debug("Group could not be unshared, likely was never shared to begin with")
+            pass
 
     def _process_users(
-        self, group: str, users_to_set_by_username: dict, enforce_group_members: bool
+        self,
+        users_to_set_by_username: dict,
+        enforce_group_members: bool,
+        keep_bots: bool,
+        group: Group,
     ):
         # group users before by username
         # (note: we DON'T get inherited users as we don't manage them at this level anyway)
-        users_before = self.gitlab.get_group_members(group, with_inherited=False)
-        debug("Group members BEFORE: %s", users_before)
-        users_before_by_username = dict()
-        for user in users_before:
-            users_before_by_username[user["username"]] = user
+        users_before = self.get_group_members(group)
+
+        debug("Group members BEFORE: %s", users_before.keys())
 
         if users_to_set_by_username:
             # group users to set by access level
@@ -171,45 +200,94 @@ class GroupMembersProcessor(AbstractProcessor):
                         else None
                     )
 
-                    if user in users_before_by_username:
-                        access_level_before = users_before_by_username[user][
-                            "access_level"
-                        ]
-                        expires_at_before = users_before_by_username[user]["expires_at"]
+                    member_role_id_or_name = (
+                        users_to_set_by_username[user]["member_role"]
+                        if "member_role" in users_to_set_by_username[user]
+                        else None
+                    )
+                    if member_role_id_or_name:
+                        member_role_id_to_set = self.gl.get_member_role_id_cached(
+                            member_role_id_or_name, group.get_id()
+                        )
+                    else:
+                        member_role_id_to_set = None
+
+                    common_username = user.lower()
+                    user_id = self.gl.get_user_id(user)
+
+                    if common_username in users_before:
+                        group_member: GroupMember = group.members.get(user_id)
+
+                        user_before = users_before[common_username]
+                        access_level_before = user_before.access_level
+                        expires_at_before = user_before.expires_at
+                        if hasattr(user_before, "member_role"):
+                            member_role_id_before = user_before.member_role["id"]
+                        else:
+                            member_role_id_before = None
 
                         if (
                             access_level_before == access_level_to_set
                             and expires_at_before == expires_at_to_set
+                            and member_role_id_before == member_role_id_to_set
                         ):
                             debug(
                                 "Nothing to change for user '%s' - same config now as to set.",
-                                user,
+                                common_username,
                             )
                         else:
                             debug(
                                 "Editing user '%s' membership to change their access level or expires at.",
-                                user,
-                            )
-                            self.gitlab.edit_member_of_group(
-                                group, user, access_level_to_set, expires_at_to_set
+                                common_username,
                             )
 
+                            group_member.access_level = access_level_to_set
+                            group_member.expires_at = expires_at_to_set
+                            group_member.member_role_id = member_role_id_to_set
+                            group_member.save()
+
                     else:
-                        debug("Adding user '%s' who previously was not a member.", user)
-                        self.gitlab.add_member_to_group(
-                            group, user, access_level_to_set, expires_at_to_set
+                        debug(
+                            "Adding user '%s' who previously was not a member.",
+                            common_username,
+                        )
+                        group.members.create(
+                            {
+                                "user_id": user_id,
+                                "access_level": access_level_to_set,
+                                "expires_at": expires_at_to_set,
+                                "member_role_id": member_role_id_to_set,
+                            }
                         )
 
         if enforce_group_members:
             # remove users not configured explicitly
             # note: only direct members are removed - inherited are left
-            users_not_configured = set(
-                [user["username"] for user in users_before]
-            ) - set(users_to_set_by_username.keys())
+            users_not_configured = set(users_before.keys()) - set(
+                [username.lower() for username in users_to_set_by_username.keys()]
+            )
             for user in users_not_configured:
+                if keep_bots and self.gitlab.get_user_by_name(user)["bot"]:
+                    debug(
+                        f"Will not remove bot user '{user}' as the 'keep_bots' option is true."
+                    )
+                    continue
                 debug("Removing user '%s' who is not configured to be a member.", user)
-                self.gitlab.remove_member_from_group(group, user)
+                user_id = self.gl.get_user_id(user)
+                try:
+                    group.members.delete(user_id)
+                except GitlabDeleteError as error:
+                    warning(f"Member could not be deleted: ", error)
+                    pass
         else:
             debug("Not enforcing group members.")
 
-        debug("Group members AFTER: %s", self.gitlab.get_group_members(group))
+        debug("Group members AFTER: %s", group.members.list(get_all=True))
+
+    @staticmethod
+    def get_group_members(group) -> dict:
+        members = group.members.list(get_all=True)
+        users = {}
+        for member in members:
+            users[member.username.lower()] = member
+        return users
