@@ -6,7 +6,7 @@ from tests.acceptance import (
 
 
 class TestTokenFromConfig:
-    def test__token_no_quotes(self, gitlab, group_and_project, token_from_env_var):
+    def test__token_no_quotes(self, project, token_from_env_var):
         config = f"""
         gitlab:
           token: {token_from_env_var}
@@ -15,9 +15,9 @@ class TestTokenFromConfig:
           placeholder:
         """
 
-        run_gitlabform(config, group_and_project)
+        run_gitlabform(config, project)
 
-    def test__token_single_quotes(self, gitlab, group_and_project, token_from_env_var):
+    def test__token_single_quotes(self, project, token_from_env_var):
         config = f"""
         gitlab:
           token: '{token_from_env_var}'
@@ -26,9 +26,9 @@ class TestTokenFromConfig:
           placeholder:
         """
 
-        run_gitlabform(config, group_and_project)
+        run_gitlabform(config, project)
 
-    def test__token_double_quotes(self, gitlab, group_and_project, token_from_env_var):
+    def test__token_double_quotes(self, project, token_from_env_var):
         config = f"""
         gitlab:
           token: "{token_from_env_var}"
@@ -37,4 +37,4 @@ class TestTokenFromConfig:
           placeholder:
         """
 
-        run_gitlabform(config, group_and_project)
+        run_gitlabform(config, project)
